//-----------------------------------------------------------------------------
// Program     : Wecker
// Beschreibung: Wecker, welcher die Zeit ueber NTP holt und auf LED-Display
//               anzeigt.
//
// Author      : Daniel Buergin
//-----------------------------------------------------------------------------
// Aenderungen : 
//-----------------------------------------------------------------------------
//
// Orignalcode von (V 1.0.87):
//
// Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
// Author:         Radig Ulrich
// Remarks:        
// known Problems: none
// Version:        24.10.2007
// Description:    Webserver uvm.
//-----------------------------------------------------------------------------
// Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
// GNU General Public License, wie von der Free Software Foundation verffentlicht, 
// weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
// (nach Ihrer Option) jeder spteren Version. 
//
// Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
// da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
// sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
// FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 
//
// Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
// Programm erhalten haben. 
// Falls nicht, schreiben Sie an die Free Software Foundation, 
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
//-----------------------------------------------------------------------------

#include <avr/io.h>
#include "config.h"
#include "usart.h"
#include "networkcard/enc28j60.h"
#include "networkcard/rtl8019.h"
#include "stack.h"
#include "timer.h"
#include "wol.h"
#include "httpd.h"
#include "cmd.h"
#include "telnetd.h"
#include "ntp.h"
#include "base64.h"
#include "http_get.h"
#include "datetime.h"
#include "pcf8583.h"
#include "max7219.h"
#include "analog.h"
#include "syslog/syslog.h"
#include "leds_switches.h"

//----------------------------------------------------------------------------
// Main Routine
//----------------------------------------------------------------------------

int main(void)
{  
	// Konfiguration der Ausgaenge

	LEDS_DDR = (1 << LED_WECKZEIT_1) | (1 << LED_WECKZEIT_2) | (1 << LED_WECKZEIT_3) ;

	// Konfiguration der Eingaenge
	
	SW_DDR &= ~ ((1 << SW_WECKZEIT_1) | 
	             (1 << SW_WECKZEIT_2) | 
				 (1 << SW_WECKZEIT_3) | 
				 (1 << SW_SET_WECKZEIT) | 
				 (1 << SW_ENCODER)) ;

	// Alle LED's ausschalten
	LED_WECKZEIT_1_OFF();
	LED_WECKZEIT_2_OFF();
	LED_WECKZEIT_3_OFF();

	DDRC = OUTC;
	DDRD = OUTD;
	
    unsigned long a;
    unsigned long b;
	char disp[60];

    usart_init(BAUDRATE); // setup the UART
	
	#if USE_ADC
		ADC_Init();
	#endif
	
	usart_write("\n\rSystem Ready\n\r");
    usart_write("Compiliert am "__DATE__" um "__TIME__"\r\n");
    usart_write("Compiliert mit GCC Version "__VERSION__"\r\n");
	for(a=0;a<1000000;a++){asm("nop");};

	//Applikationen starten
	stack_init();
	httpd_init();
	telnetd_init();
	
	//Ethernetcard Interrupt enable
	ETH_INT_ENABLE;
	
	//Globale Interrupts einschalten
	sei(); 
	
	// Etwas warten damit sich er Ethernet Stack etablieren kann
	for(a=0;a<5000000;a++){asm("nop");};

	#if USE_SYSLOG
		syslog_send("<4>Booting....");
	#endif //syslog

	max7219_Init(1,1,0xff,0x03,7);
	max7219_Init(2,1,0xff,0x03,7);

	initRtc();

	for(a=0;a<5000000;a++){asm("nop");};

	#if USE_NTP
		ntp_init();
		ntp_request();
	#endif //USE_NTP

	a = time;
	b = 0;

	while(1)
	{
		#if USE_ADC
		  ANALOG_ON;
		#endif

	    eth_get_data();
		
		if (usart_status.usart_ready)
		{
			if(extract_cmd(&usart_rx_buffer[0]))
			{
				usart_write("Ready\r\n\r\n");
			}
			else
			{
				usart_write("ERROR\r\n\r\n");
			}
			usart_status.usart_ready =0;
		}
		
		#if USE_NTP
		if(!ntp_timer){
			ntp_timer = NTP_REFRESH;
			ntp_request();
		}
		#endif //USE_NTP
		
		if (time_sw)
		{
			time_sw = 0;
			
			// Timestamp in reale Zeit umrechnen
			
			readRtc(&rtc_time);

			// Timebuffer fuellen

			disp_memory[7] = rtc_time.hour / 10;
			disp_memory[6] = rtc_time.hour % 10;
			disp_memory[5] = 0x0a;
			disp_memory[4] = rtc_time.minute / 10;
			disp_memory[3] = rtc_time.minute % 10;
			disp_memory[2] = 0x0a;
			disp_memory[1] = rtc_time.second / 10;
			disp_memory[0] = rtc_time.second % 10;

			max7219_Display(1);

			// Datebuffer fuellen
			
			disp_memory[7] = rtc_time.day / 10;
			// disp_memory2[6] = (rtc_time.day % 10) | (1<<DPBIT);
			disp_memory[6] = rtc_time.day % 10;
			disp_memory[5] = 0x0a;
			disp_memory[4] = rtc_time.month / 10;
			disp_memory[3] = rtc_time.month % 10;
			disp_memory[2] = 0x0a;
			disp_memory[1] = ((rtc_time.year / 10) % 100); // Die zweite Stelle in der Jahrzahl ermitteln
			disp_memory[0] = rtc_time.year % 1000; // Die letzte Stelle in der Jahrzahl ermitteln

			max7219_Display(2);

			if (SW_PIN & (1 << SW_SET_WECKZEIT))
			{
				if (SW_PIN & (1 << SW_WECKZEIT_1))
				{
					LEDS_PORT = LEDS_PIN ^ (1 << LED_WECKZEIT_1);
				}

				if (SW_PIN & (1 << SW_WECKZEIT_2))
				{
					LEDS_PORT = LEDS_PIN ^ (1 << LED_WECKZEIT_2);
				}

				if (SW_PIN & (1 << SW_WECKZEIT_3))
				{
					LEDS_PORT = LEDS_PIN ^ (1 << LED_WECKZEIT_3);
				}
			}

		}

		if ((a + 60) <= time)
		{
			b++;
			a = time;
			sprintf(disp,"I'am alive: %lu time: %lu\r\n",b,a);
			usart_write_str(disp);
		}

		//USART Daten fr Telnetanwendung?
		telnetd_send_data();
    }
		
return(0);
}
