//***************************************************************************************
//* Driver for 2 x Maxim MAX7219 8 x 7-Segment Display (SPI)
//*
//* by D. Buergin (daniel.buergin@bluewin.ch)
//****************************************************************************************

#include "max7219.h"

// Display Memory fuer ersten max7219

unsigned int disp_memory[8] =
{
	BLANK,
	BLANK,
	BLANK,
	BLANK,
	BLANK,
	BLANK,
	BLANK,
	BLANK
};

//****************************************************************************************
//* Ein Byte auf einen MAX7219 schreiben
//****************************************************************************************

void max7219_WriteByte (unsigned char data_out)
{
	unsigned char loop, mask; //0x80 = 10000000
	for (loop=0,mask=0x80;loop<8;loop++, mask=mask>>1)
	{
		// Clock Line runternehmen

		MAX7219_PORT &= ~(1<<MAX7219_PORT_CLK);

		// Das Bit rausschreiben

		if (data_out & mask)
		{
			MAX7219_PORT |= (1<<MAX7219_PORT_DIN);
		}
		else
		{
			MAX7219_PORT &= ~(1<<MAX7219_PORT_DIN);
		}
			
		// Die Clockline wieder raufnehmen

		MAX7219_PORT |= (1<<MAX7219_PORT_CLK);
	}
	MAX7219_PORT &= ~(1<<MAX7219_PORT_CLK);
}

//****************************************************************************************
//* Ein Adress- und ein Datenbyte rausschreiben
//* Diese Funktion muss erweitert werden, sollte es mehr als zwei MAX7219 haben
//****************************************************************************************

void max7219_Write (unsigned int max, unsigned char address, unsigned char data)
{
	if (max == 1)
	{
		// Daten fuer den zweiten MAX7219
		max7219_WriteByte(0x0);
		max7219_WriteByte(0x0);
		// Daten fuer den ersten MAX7219
		max7219_WriteByte(address);
		max7219_WriteByte(data);
	}
	else
	{
		// Daten fuer den zweiten MAX7219
		max7219_WriteByte(address);
		max7219_WriteByte(data);
		// Daten fuer den ersten MAX7219
		max7219_WriteByte(0x0);
		max7219_WriteByte(0x0);
	}

	// Nun die LOAD Line pulsen, damit uebernehmen die MAX7219 die Daten

	MAX7219_PORT |= (1<<MAX7219_PORT_LOAD);
	MAX7219_PORT &= ~(1<<MAX7219_PORT_LOAD);
}

//****************************************************************************************
//* Den disp_memory[8] Buffer ausgeben (egal was drin ist)
//* Input: max = Auf welchen MAX7219 geschrieben werden soll (1 oder 2)
//****************************************************************************************

void max7219_Display(unsigned int max)
{
	unsigned int cnt = 0;

	for (cnt = 8; cnt > 0; cnt--)
	{
		max7219_Write(max,cnt,disp_memory[cnt-1]);	
	}
}

//****************************************************************************************
//* Den entsprechene MAX7219 initialisieren
//****************************************************************************************

void max7219_Init (unsigned int max, unsigned int shutdown, unsigned int decode_mode, unsigned int bright, unsigned int scan)
{
	unsigned char loop;

	// Set all DDR Register to Output

	MAX7219_DDR |= (1<<MAX7219_PORT_CLK);
	MAX7219_DDR |= (1<<MAX7219_PORT_DIN);
	MAX7219_DDR |= (1<<MAX7219_PORT_LOAD);

	max7219_Write(max,0x0c, shutdown);    // 0x01 = Shutdown aus
	max7219_Write(max,0x09, decode_mode); // 0xff = Decode-Mode: alles auf Code B
	max7219_Write(max,0x0a, bright);      // 0x03 = Intensitaet 0 bei hellen LED's 3 bei mittleren LED's (0x0 - 0xf)
	max7219_Write(max,0x0b, scan);        // 0x07 = 8 Digits 0x00 gleich 1 Digit

	// "-" auf allen Anzeigen

	for (loop = 1; loop < (scan + 2); loop++)
	{
		max7219_Write(max,loop, 0x0a); // 0x0a = "-"
	}
}
