#ifndef MAX7219_H
#define MAX7219_H

//****************************************************************************************
//* Driver for Maxim MAX7219 8 x 7-Segment Display (SPI)
//*
//* by D. Buergin (daniel.buergin@bluewin.ch)
//****************************************************************************************
//* Verbindung auf ATmega128:
//*
//* ATmega128            MAX7219
//* ----------------------------------
//* Port D Pin 5 ----->  Pin 13 (CLK)
//* Port D Pin 6 ----->  Pin 12 (LOAD)
//* Port D Pin 7 ----->  Pin 1  (DIN)
//****************************************************************************************

#include <avr/io.h>

// Port of MAX7219

#define MAX7219_PORT        PORTD
#define MAX7219_DDR         DDRD

// CLK Port of MAX7219

#define MAX7219_PORT_CLK	PD7

// LOAD Port of MAX7219

#define MAX7219_PORT_LOAD	PD6

// DIN Port of MAX7219

#define MAX7219_PORT_DIN	PD5

// Character Table
// 0 - 9 is 0x0 - 0x9 ;-)
// For DP, set Bit D7

#define DP    0x8f // Blank plus DP
#define LINE  0x0a
#define E     0x0b
#define H     0x0c
#define L     0x0d
#define P     0x0e
#define BLANK 0x0f
#define DPBIT 0x07

extern unsigned int disp_memory[];

void max7219_WriteByte (unsigned char data_out);
void max7219_Write (unsigned int max, unsigned char address, unsigned char data);
void max7219_Display (unsigned int max);
void max7219_Init (unsigned int max, unsigned int shutdown, unsigned int decode_mode, unsigned int bright, unsigned int scan);

#endif
